Package: bigstep
Type: Package
Title: Stepwise Selection for Large Data Sets
Version: 0.7.4
Date: 2017-4-5
Author: Piotr Szulc
Maintainer: Piotr Szulc <piotr.michal.szulc@gmail.com>
Description: Selecting linear and generalized linear models for large data sets
    using modified stepwise procedure and modern selection criteria (like
    modifications of Bayesian Information Criterion). Selection can be
    performed on data which exceed RAM capacity. Special selection strategy is
    available, faster than classical stepwise procedure.
License: GPL-3
Depends: R (>= 3.2.2)
Imports: stats, methods, utils, RcppEigen, speedglm, bigmemory,
        R.utils, matrixStats
Suggests: testthat, devtools
RoxygenNote: 5.0.1
NeedsCompilation: no
Packaged: 2017-04-05 20:50:28 UTC; s
Repository: CRAN
Date/Publication: 2017-04-05 21:57:37 UTC
Built: R 3.4.1; ; 2018-10-20 19:55:57 UTC; windows
