Package: bigstep
Type: Package
Title: Stepwise Selection for Large Data Sets
Version: 1.1.1
Date: 2023-5-11
Authors@R: person("Piotr", "Szulc", email = "piotr.michal.szulc@gmail.com", role = c("aut", "cre"))
Description: Selecting linear and generalized linear models for large data sets
    using modified stepwise procedure and modern selection criteria (like
    modifications of Bayesian Information Criterion). Selection can be 
    performed on data which exceed RAM capacity. Bogdan et al., (2004)
    <doi:10.1534/genetics.103.021683>.
License: GPL-3
URL: https://github.com/pmszulc/bigstep
BugReports: https://github.com/pmszulc/bigstep/issues
Depends: R (>= 3.5.0)
Imports: bigmemory, magrittr, matrixStats, R.utils, RcppEigen,
        speedglm, stats, utils
Suggests: devtools, knitr, rmarkdown, testthat
Encoding: UTF-8
RoxygenNote: 7.2.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2023-05-11 14:03:12 UTC; Piotr
Author: Piotr Szulc [aut, cre]
Maintainer: Piotr Szulc <piotr.michal.szulc@gmail.com>
Repository: CRAN
Date/Publication: 2023-05-13 23:12:03 UTC
Built: R 4.4.0; ; 2024-05-01 00:00:45 UTC; unix
