# Generated by roxygen2: do not edit by hand

S3method(summary,big)
export("%>%")
export(aic)
export(attach.big.matrix)
export(backward)
export(bic)
export(fast_forward)
export(forward)
export(get_model)
export(maic)
export(maic2)
export(mbic)
export(mbic2)
export(multi_backward)
export(prepare_data)
export(read.big.matrix)
export(reduce_matrix)
export(stepwise)
importFrom(RcppEigen,fastLmPure)
importFrom(bigmemory,attach.big.matrix)
importFrom(bigmemory,read.big.matrix)
importFrom(magrittr,"%>%")
importFrom(speedglm,speedglm.wfit)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,poisson)
importFrom(stats,sd)
