# Generated by roxygen2: do not edit by hand

export(aic)
export(bic)
export(combineBigMatrices)
export(fitLinear)
export(fitLogistic)
export(fitPoisson)
export(maic)
export(maic2)
export(mbic)
export(mbic2)
export(selectModel)
export(singleTests)
importFrom(R.utils,doCall)
importFrom(RcppEigen,fastLmPure)
importFrom(bigmemory,as.big.matrix)
importFrom(bigmemory,read.big.matrix)
importFrom(matrixStats,colSds)
importFrom(methods,is)
importFrom(speedglm,speedglm.wfit)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
